require "prefabutil"

---------------Config data
local name = "Mandrake Tree" 
local folder = KnownModIndex:GetModActualName(name)

local FLIESPERDAY=GetModConfigData("FLIESPERDAY", folder)	
local MAXFLIES=GetModConfigData("MAXFLIES", folder)	
local MANDRAKESPER5DAYS=GetModConfigData("MANDRAKESPER5DAYS", folder)	
local MAXMANDRAKES=GetModConfigData("MAXMANDRAKES", folder)		
local childtype1="fireflies"
local childtype2="mandrake_planted"

---------------

local assets =
{
	Asset("ATLAS", "images/inventoryimages/mandraketree.xml"),
	Asset("IMAGE", "images/inventoryimages/mandraketree.tex"),
	Asset("ANIM", "anim/mandraketree.zip"),  
}
---------animations------
local anim={}
anim.idle = "idle_flies"
anim.hit = anim.idle
anim.place ="idle"
	  
local prefabs =
{
    childtype1,
	childtype2,
	
    "collapse_small",
}	  

local function onhammered(inst, worker)
	if inst:HasTag("fire") and inst.components.burnable then
		inst.components.burnable:Extinguish()
	end
	inst.components.lootdropper:DropLoot()
	SpawnPrefab("collapse_small").Transform:SetPosition(inst.Transform:GetWorldPosition())
	inst.SoundEmitter:PlaySound("dontstarve/common/destroy_wood")
	inst:Remove()
end

local function onhit(inst, worker)
	if not inst:HasTag("burnt") then 
		inst.AnimState:PlayAnimation(inst.anims.hit)
		inst.AnimState:PushAnimation(inst.anims.idle, false)
	end
end

local function OnSave(inst, data)
	if inst:HasTag("burnt") or inst:HasTag("fire") then
		data.burnt = true
	end
end

local function OnLoad(inst, data)
	if data and data.burnt then
		inst.components.burnable.onburnt(inst)
	end
end
------plantation-----
local function onbuilt(inst)
	inst.AnimState:PlayAnimation(anim.place)
	inst.AnimState:PushAnimation(anim.idle, false)
end

-------fireflies fonction de croissance
local function PlantFlies(inst)
	local minrad=1
	local maxrad=4
	
	--inst exists
	if not inst then return end
	local pt = inst:GetPosition()
	
	------fireflies spawn-------------
	--not too many children
	local ents = TheSim:FindEntities(pt.x, pt.y, pt.z, maxrad)
	local count1=0
	for k,v in pairs(ents) do
		if v.prefab == childtype1 then
			count1 = count1 + 1
		end
	end
    if count1>=MAXFLIES then return end
	
	--Find spot for new child and spawn
	local angle=math.random()*2*PI
	local radius=math.random()*(maxrad-minrad)+minrad
	local offset, check_angle, deflected=FindWalkableOffset(pt, angle, radius, 6, false, false)
	if(not check_angle) then return end
	angle=check_angle
	pt.x=pt.x+radius*math.cos(angle)
	pt.z=pt.z-radius*math.sin(angle)
	
	SpawnPrefab(childtype1).Transform:SetPosition(pt:Get())
end

local function PlantMandrakes(inst)
	local minrad=1
	local maxrad=4
	
	--inst exists
	if not inst then return end
	local pt = inst:GetPosition()
	

	--not too many children
	local ents = TheSim:FindEntities(pt.x, pt.y, pt.z, maxrad)
	local count=0
	for k,v in pairs(ents) do
		if v.prefab == childtype2 then
			count = count + 1
		end
	end
	
    if count>=MAXMANDRAKES then return end
	
	--Find spot for new child and spawn
	local angle=math.random()*2*PI
	local radius=math.random()*(maxrad-minrad)+minrad
	local offset, check_angle, deflected=FindWalkableOffset(pt, angle, radius, 8, false, false)
	if(not check_angle) then return end
	angle=check_angle
	pt.x=pt.x+radius*math.cos(angle)
	pt.z=pt.z-radius*math.sin(angle)
	
	SpawnPrefab(childtype2).Transform:SetPosition(pt:Get())
end
--------placement de l'arbre
local function saveanims(inst)
	if inst.anims==nil then
		inst.anims = {}
	end
	
	inst.anims.idle = anim.idle
	inst.anims.hit = anim.hit
	inst.anims.place = anim.place
end

local function fn()
	---------------Entity details
	local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	inst.entity:AddMiniMapEntity()
	inst.entity:AddNetwork()--DST only
	inst.entity:AddLight()              
	MakeObstaclePhysics(inst, .5)
	
	---------------Animation
	inst.MiniMapEntity:SetIcon("mandraketree.tex")
	inst.AnimState:SetBank("mandraketree")		
    inst.AnimState:SetBuild("mandraketree")
	inst.AnimState:PlayAnimation(anim.idle)
	
	---------------Tags
	inst:AddTag("structure")
	inst:AddTag("playerowned")
	
	---------------????	--DST only
	MakeSnowCoveredPristine(inst)
	inst.entity:SetPristine()
	if not TheWorld.ismastersim then
		return inst
	end 
	
	---------------Growing details
		
	local growth_stages = {
		{name="mandrake_planted",time = function(inst) return TUNING.TOTAL_DAY_TIME*5/MANDRAKESPER5DAYS end, fn = function(inst) PlantMandrakes(inst) end},
		{name="fireflies",time = function(inst) return TUNING.TOTAL_DAY_TIME/FLIESPERDAY end, fn = function(inst) PlantFlies(inst) end},
	}
	
	inst:AddComponent("growable")
	inst.components.growable.stages = growth_stages
	inst.components.growable.stage = 1
	inst.components.growable.loopstages = true
	inst.components.growable:StartGrowing()
	
	---------------Components
	inst:AddComponent("inspectable")
	inst:AddComponent("lootdropper")
	inst:AddComponent("workable")
	inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
	inst.components.workable:SetWorkLeft(4)
	inst.components.workable:SetOnFinishCallback(onhammered)
	inst.components.workable:SetOnWorkCallback(onhit)
	
	--Light Radius
	
	inst.Light:Enable(true)
	inst.Light:SetRadius(1)
	inst.Light:SetFalloff(.5)
	inst.Light:SetIntensity(.5)
	inst.Light:SetColour(0/255,255/255,255/255)
        --------------------      
    saveanims(inst)
		
	MakeHauntableWork(inst)		--DST only
	MakeSnowCovered(inst)
	inst:ListenForEvent("onbuilt", onbuilt)
	
	MakeMediumBurnable(inst, nil, nil, true)
	MakeLargePropagator(inst)
	
	inst.OnSave = OnSave
	inst.OnLoad = OnLoad
	---------------------        
    return inst 
end
		
return	Prefab("common/objects/mandraketree", fn, assets, prefabs),     	 
    MakePlacer("common/mandraketree_placer", "mandraketree", "mandraketree", "idle")
       